/**
 * This message buffer is responsible for smooth display of logging messages in the GUI
 * @author Johannes Dellert <jdellert@sfs.uni-tuebingen.de>
 * @version 0.1, November 2007
 */

public class Drop
{
    //Message sent from producer to consumer.
    private String message;
    //True if consumer should wait for producer to send message, false
    //if producer should wait for consumer to retrieve message.
    private boolean empty = true;

    /**
     * during a GUI update, get the new message arriving in the buffer
     * @return the next message from the buffer
     */
    public synchronized String take() 
    {
        //Wait until message is available.
        while (empty) 
        {
            try 
            {
                wait();
            } 
            catch (InterruptedException e) {}
        }
        //Toggle status.
        empty = true;
        //Notify producer that status has changed.
        notifyAll();
        return message;
    }

    /**
     * used by backends to hand on their logging and debugging output
     * @param message - the message to be printed
     */
    public synchronized void put(String message) 
    {
        //Wait until message has been retrieved.
        while (!empty) 
        {
            try 
            { 
                wait();
            } 
            catch (InterruptedException e) {}
        }
        //Toggle status.
        empty = false;
        //Store message.
        this.message = message;
        //Notify consumer that status has changed.
        notifyAll();
    }
}
